@@ -8,7 +8,7 @@ __DATA__
 # Constants in a block
 # CONTEXT no warnings;
 {
-    '???';
+    '?unrecoverable constant?';
     2;
 }
 ####
@@ -18,7 +18,7 @@ __DATA__
 (1,2,3);
 0;
 >>>>
-'???', '???', '???';
+'?unrecoverable constant?', '?unrecoverable constant?', '?unrecoverable constant?';
 0;
 ####
 # Lexical and simple arithmetic
@@ -28,11 +28,9 @@ my $test;
 my $test;
 $test /= 2 if ++$test;
 ####
-# SKIP ?1
 # list x
 -((1, 2) x 2);
 ####
-####
 # lexical and package scalars
 my $x;
 print $main::x;
@@ -62,12 +60,10 @@ glob $foo;
 glob $foo;
 ####
 # block
-# SKIP ?1
-{ my $x; }
+{ my $x }
 ####
-# SKIP ?1
 # while 1
-while (1) { my $k; }
+while (1) { my $k }
 ####
 # constants as method names
 'Foo'->bar('orz');
@@ -78,7 +74,7 @@ while (1) { my $k; }
 # SKIP ?$] < 5.010 && "say not implemented on this Perl version"
 # CONTEXT use feature ':5.10';
 # say
-say 'foo';
+say 'foo'
 ####
 # SKIP ?$] < 5.010 && "say not implemented on this Perl version"
 # CONTEXT use 5.10.0;
@@ -88,7 +84,7 @@ say 'foo';
 # SKIP ?$] < 5.010 && "say not implemented on this Perl version"
 # say with use 5.10.0
 use 5.10.0;
-say 'foo';
+say 'foo'
 >>>>
 no feature ':all';
 use feature ':5.10';
@@ -106,19 +102,19 @@ say 'foo';
 # CONTEXT use feature ':5.10';
 # say with use 5.10.0 in the context of use feature
 use 5.10.0;
-say 'foo';
+say 'foo'
 >>>>
 no feature ':all';
 use feature ':5.10';
-say 'foo';
+say 'foo'
 ####
-# SKIP ?$] < 5.010 && "say not implemented on this Perl version"
-# CONTEXT use 5.10.0;
+# SKIP ?1
+CONTEXT use 5.10.0;
 # say with use feature ':5.10' in the context of use 5.10.0
 use feature ':5.10';
 say 'foo';
 >>>>
-say 'foo';
+say 'foo'
 ####
 # SKIP ?$] < 5.015 && "__SUB__ not implemented on this Perl version"
 # CONTEXT use feature ':5.15';
@@ -257,7 +253,6 @@ pop();
 # shift optimisation
 pop @_;
 ####
-# SKIP ?1
 #[perl #20444]
 "foo" =~ (1 ? /foo/ : /bar/);
 "foo" =~ (1 ? y/foo// : /bar/);
@@ -267,7 +262,7 @@ pop @_;
 'foo' =~ ($_ =~ /foo/);
 'foo' =~ ($_ =~ tr/fo//);
 'foo' =~ ($_ =~ tr/fo//r);
-'foo' =~ ($_ =~ s/foo//);
+'foo' =~ ($_ =~ s/foo//)
 # ####
 # # The fix for [perl #20444] broke this.
 # 'foo' =~ do { () };
@@ -306,7 +301,7 @@ my @x;
 @x = ($#{`}, $#{~}, $#{!}, $#{@}, $#{$}, $#{%}, $#{^}, $#{&}, $#{*});
 @x = ($#{(}, $#{)}, $#{[}, $#{{}, $#{]}, $#{}}, $#{'}, $#{"}, $#{,});
 @x = ($#{<}, $#{.}, $#{>}, $#{/}, $#{?}, $#{=}, $#+, $#{\}, $#{|}, $#-);
-@x = ($#{;}, $#{:}, $#{1}), $#_;
+@x = ($#{;}, $#{:}, $#{1}), $#_
 # ####
 # # [perl #86060] $( $| $) in regexps need braces
 # /${(}/;
@@ -362,7 +357,6 @@ $a++;
 # print $_;
 ####
 # Precedence conundrums with argument-less function calls
-# SKIP ?1
 () = (eof) + 1;
 () = (return) + 1;
 () = (return, 1);
@@ -378,7 +372,6 @@ pipe local *FH, local *FH;
 require 'a' . $1;
 ####
 # 'my' works with padrange op
-# SKIP ?1
 my($z, @z);
 my $m1;
 $m1 = 1;
@@ -393,7 +386,6 @@ my($m7, undef, $m8) = (1, 2, 3);
 ($m7, undef, $m8) = (1, 2, 3);
 ####
 # 'state' works with padrange op
-# SKIP ?1
 # CONTEXT no strict; use feature 'state';
 state($z, @z);
 state $s1;
@@ -410,6 +402,7 @@ state($s3, $s4);
 ($s7, undef, $s8) = (1, 2, 3);
 ####
 # slices with padrange
+# SKIP $?$] >= 5.024
 my($a, $b);
 my(@x, %y);
 @x = @x[$a, $b];
@@ -460,5 +453,4 @@ tr/X//s;
 tr/X//r;
 ####
 use feature 'unicode_strings';
-# SKIP ?1
 s/X//d;
